
do

	bullets = {
		list = {},
		count = 0,
		types = {},
		globalCount = 0,
	}


	function bullets.load()
		-- setup types
		bullets.addType(love.graphics.newImage("media/shot1.png"), 0.4, "shot1.wav", 0.25, 1000.0, 10, 1.0)
		bullets.addType(love.graphics.newImage("media/shot2.png"), 0.2, "shot2.wav", 0.4, 450, 15, 2.2, function(t,b) return 0, ((b.typeId % 2 > 0) and 1 or -1) * 50*math.sin(t*7) end)
		bullets.addType(love.graphics.newImage("media/shot3.png"), 0.6, nil, 2.5, 80, 80, 2.0, function(t,b) return t*t*t*80*(1.0 + ((b.typeId*0.53231)%0.1)),0 end).explosionScale = 3.0
		bullets.addType(love.graphics.newImage("media/shot4.png"), 0.36, "shot4.wav", 0.1, 1250.0, 6, 0.7, function(t,b) return 0,90*t*(((0.743164*b.typeId) % 1) - ((0.3115*b.typeId*b.typeId) % 1)) end)
	end

	function bullets.clear()
		bullets.list = {}
		bullets.count = 0
	end

	function bullets.addType(sprite, scale, sound, delay, speed, damage, lifetime, offsetFunction)
		local obj = {
			sprite = sprite,
			scale = scale,
			sound = sound,
			delay = delay,
			speed = speed,
			damage = damage,
			lifetime = lifetime,
			offsetter = offsetFunction,
			globalCount = 0, --how many bullets of that type have been created
			explosionScale = 1,
		}
		bullets.types[#bullets.types+1] = obj
		return obj
	end

	function bullets.create(ship, offset, type)
		bullets.globalCount = bullets.globalCount + 1
		type.globalCount = type.globalCount + 1
		local angle = ship.angle + offset[2]*math.pi/180.0
		local sin = math.sin(angle)
		local cos = math.cos(angle)
		local obj = {
			kind = "bullet",
			ship = ship,
			type = type,
			x = ship.x,
			y = ship.y,
			basex = ship.x + offset[3]*cos - offset[4]*sin,
			basey = ship.y + offset[3]*sin + offset[4]*cos,
			angle = angle,
			vx = type.speed * cos + ship.vx*0.8,
			vy = type.speed * sin + ship.vy*0,
			sin = sin,
			cos = cos,
			age = 0,
			id = bullets.globalCount,
			typeId = type.globalCount,
		}
		table.insert(bullets.list, obj)
		bullets.count = bullets.count + 1
		-- sound
		if type.sound then
			lush.play(type.sound, {volume=0.1})
		end
		return obj
	end

	function bullets.deleteID(id)
		bullets.count = bullets.count - 1
		table.remove(bullets.list, id)
	end

	function bullets.delete(obj)
		for i = 1,bullets.count do
			if bullets.list[i] == obj then
				bullets.deleteID(i)
				return true
			end
		end
		return false
	end

	function bullets.update()
		for b = bullets.count,1,-1 do
			bullets.updateInstance(bullets.list[b])
		end
	end

		function bullets.updateInstance(b)
			-- Lifetime
			b.age = b.age + gameState.dt
			if b.age > b.type.lifetime or b.x > portal.x then
				bullets.delete(b)
				return
			end
			-- Move
			b.basex = b.basex + b.vx * gameState.dt
			b.basey = b.basey + b.vy * gameState.dt
			if gameState.outside(b.basey) then
				bullets.delete(b)
				return
			end
			-- Offset
			if b.type.offsetter then
				local off1, off2 = b.type.offsetter(b.age, b)
				b.x = b.basex + (b.cos * off1 - b.sin * off2)
				b.y = b.basey + (b.sin * off1 + b.cos * off2)
			else
				b.x = b.basex
				b.y = b.basey
			end
			-- collide with ships
			if enemy.sphereCollision(b.x, b.y, 5) then
				enemy.damage(enemy.collider, b.type.damage)
				explosion.create(b.x, b.y, 0.3*b.type.explosionScale, 1.4, 0.8)
				bullets.delete(b)
				return
			end
			-- collide with asteroids
			if asteroid.sphereCollision(b.x, b.y, 5) then
				asteroid.damage(asteroid.collider, b.type.damage)
				explosion.create(b.x, b.y, 0.4*b.type.explosionScale, 1.6, 0.6)
				bullets.delete(b)
				return
			end
		end

	function bullets.draw()
		for b = 1, bullets.count do
			bullets.drawInstance(bullets.list[b])
		end
	end

		function bullets.drawInstance(b)
			if type(b.type.sprite) == "table" then
				love.graphics.setColor(b.type.sprite)
				love.graphics.line(b.x, b.y, b.x + 30*math.cos(b.angle), b.y + 30*math.sin(b.angle))
			else
				love.graphics.setColor(255,255,255,255)
				love.graphics.draw(b.type.sprite, b.x, b.y, b.angle, b.type.scale, b.type.scale, 64, 64)
			end
		end





end