
do

	enemy = {
		list = {},
		count = 0,
		types = {},
		movements = {},
	}


	function enemy.load()
		-- create types
		enemy.addType("Weak", "media/enemy/enemy1_", 0.065, 30, 4)
		enemy.addType("Mediocre", "media/enemy/enemy2_", 0.055, 100, 4)
		-- create movements
		enemy.addMovement("straight", function(t) return -t, 0 end)
		enemy.addMovement("shift", function(t) return 0,t end)
		enemy.addMovement("wave", function(t) return 0,math.sin(t) end)
		enemy.addMovement("wavex", function(t) return math.sin(t),0 end)
		enemy.addMovement("circle", function(t) return math.sin(t),math.cos(t) end)
		enemy.addMovement("accelerate", function(t) 
			if t > 0 then return -t*t,0 end
			return 0,0
		end)
		enemy.addMovement("hardwave", function(t) return 0, 2 * (0.5 - 0.5*math.cos(PIHalf - PIHalf*math.cos(PIHalf - PIHalf*math.cos(t)))) - 1 end)
		enemy.addMovement("hardwavex", function(t) return 2 * (0.5 - 0.5*math.cos(PIHalf - PIHalf*math.cos(PIHalf - PIHalf*math.cos(t + PIHalf)))) - 1, 0 end)
	end

	function enemy.clear()
		enemy.list = {}
		enemy.count = 0
	end


	function enemy.addType(name, sprites, frameDelay, hp, frames)
		if type(sprites) == "string" then
			local list = {}
			for i = 1,frames do
				list[i] = love.graphics.newImage(sprites .. i .. ".png")
			end
			sprites = list
		end
		obj = {
			name = name,
			sprites = sprites,
			hp = hp,
			frames = #sprites,
			frameDelay = frameDelay
		}
		enemy.types[#enemy.types+1] = obj
		return obj
	end

	function enemy.addMovement(key, func)
		enemy.movements[key] = func
	end

	function enemy.create(tp, x, y, ageDelay, moves)
		if type(tp) == "number" then
			tp = enemy.types[tp]
		end
		-- valid position?
		if x > portal.x - 400 then
			return nil
		end
		-- prepare moves list
		for i = 1,#moves do
			local move = moves[i]
			if type(move[1]) == "string" then
				move[1] = enemy.movements[move[1]]
			end
			move[2] = move[2] or 100
			move[3] = move[3] or move[2]
			move[4] = move[4] or 1.0
		end
		-- Create
		local obj = {
			type = tp,
			moves = moves,
			x0 = x,
			y0 = y,
			x = x,
			y = y,
			age = -ageDelay,
			ageDelay = ageDelay,
			hp = tp.hp,
			angle = 0,
			aliveAge = nil
		}
		-- sort in
		enemy.count = enemy.count + 1
		table.insert(enemy.list, obj)
		return obj
	end

	function enemy.createGroup(tp, count, x, y, dx, dy, ageDelay, moves)
		local mid = (count+1)/2
		for i = 1,count do
			enemy.create(tp, x + dx*(i-mid), y + dy*(i-mid), (i-1)*ageDelay, copyTable(moves))
		end
	end

	function enemy.deleteID(id)
		enemy.count = enemy.count - 1
		table.remove(enemy.list, id)
	end

	function enemy.delete(obj)
		for e = 1,enemy.count do
			if enemy.list[e] == obj then
				enemy.deleteID(e)
				return true
			end
		end
		return false
	end

	function enemy.update()
		for e = enemy.count,1,-1 do
			enemy.updateInstance(enemy.list[e])
		end
	end

		function enemy.updateInstance(e)
			-- Age
			e.age = e.age + gameState.dt
			-- Near Player
			if not e.aliveAge then
				if e.x < spaceship.instance.x + 1500 then
					e.aliveAge = -e.ageDelay
					e.aliveAgeStart = e.age + e.ageDelay
				end
			else
				e.aliveAge = e.age - e.aliveAgeStart
			end
			-- movement
			e.prevx = e.x
			e.prevy = e.y
			e.offx = 0
			e.offy = 0
			if e.aliveAge then
				for i = 1,#e.moves do
					x,y = e.moves[i][1](e.aliveAge * e.moves[i][4])
					e.offx = e.offx + e.moves[i][2] * x
					e.offy = e.offy + e.moves[i][3] * y
				end
				e.x = e.x0 + e.offx
				e.y = gameState.wrapY(e.y0 + e.offy)
			end
			local vx = (e.x - e.prevx)/gameState.dt
			e.angle = 0.002*vx
			-- leaving screen
			if e.x < -500 then
				enemy.delete(e)
			end
		end

	function enemy.draw()
		love.graphics.setColor(255,255,255,255)
		for e = 1, enemy.count do
			enemy.drawInstance(enemy.list[e])
		end
	end

		function enemy.drawInstance(e)
			local frame
			if e.type.frames == 1 then
				frame = 1
			else
				frame = math.floor(gameState.time/e.type.frameDelay)
				frame = 1 + frame % e.type.frames
			end
			love.graphics.draw(e.type.sprites[frame], e.x, e.y, e.angle, 0.4, 0.4, 128, 128)
		end


	function enemy.sphereCollision(x,y,r)
		local e
		for i = 1,enemy.count do
			e = enemy.list[i]
			if collision.sphereSphere(x,y,r, e.x,e.y,20) then
				enemy.collider = e
				return e
			end
		end
		return nil
	end

	function enemy.damage(e, dmg)
		e.hp = e.hp - dmg
		if e.hp <= 0 then
			explosion.create(e.x, e.y, 1.2)
			enemy.delete(e)
		end
	end

end