PORT = "30927"

require "conf"
require "states"
require "utility"
require "gameState"
require "spaceship"
require "asteroid"
require "bullets"
require "portal"
require "enemy"
require "collision"
require "pickups"
require "background"
require "explosion"
require "levelBuilder"
require "sequences"
lush = require("lush")

function love.load()
    print("Loading all")
    gameState.load()
    print("Loading seq")
    sequences.load()
    print("Entering seq")
    setState(sequences)
	lush.setPath("media/sound/")
    lush.play("music.mp3", {looping = true, volume = 0.5})
end

function love.update(dt)
    updateDelayedCalls()
    --updateWatchedInputs()

    local state = getState()
    state.prevtime = state.time
    state.time = state.time + dt
    if state.update then state.update(dt) end
end

function love.textinput(text)
    if getState().textinput then getState().textinput(text) end
end

function love.draw()
    love.graphics.setColor(255, 255, 255, 255)
    if getState().draw then getState().draw() end
end
function love.keypressed(key, isrepeat)
    if getState().keypressed then getState().keypressed(key, isrepeat) end
end

function love.keyreleased(key)
    if getState().keyreleased then getState().keyreleased(key) end
end

function love.runnnnOrDont()
    if love.math then
        love.math.setRandomSeed(os.time())
        for i=1,3 do love.math.random() end
    end

    if love.event then
        love.event.pump()
    end

    simulationTime = love.timer.getTime()
    simulationDt = 1.0/40.0

    if love.load then love.load(arg) end

    -- Main loop
    while true do
        while simulationTime < love.timer.getTime() do
            simulationTime = simulationTime + simulationDt

            -- Process events.
            if love.event then
                love.event.pump()
                for e,a,b,c,d in love.event.poll() do
                    if e == "quit" then
                        if not love.quit or not love.quit() then
                            if love.audio then
                                love.audio.stop()
                            end
                            return
                        end
                    end
                    love.handlers[e](a,b,c,d)
                end
            end

            love.update()
        end

        lush.update()

        if love.window and love.graphics and love.window.isCreated() then
            love.graphics.clear()
            love.graphics.origin()
            if love.draw then love.draw() end
            love.graphics.present()
        end

        if love.timer then love.timer.sleep(0.001) end
    end
end


function endGame()
    love.event.push("quit")
end