
do

	spaceship = {
		instance,
		gravity = 120.0,
		weaponSpots = {
			{nil, 0, 50, 0}, --bullet type (initially nil), angle in degrees (0 being right, then clockwise), offset forward, offset left
			{nil, 0, 50, -20},
			{nil, 0, 50, 20},
			{nil, -45, 20, 0},
			{nil, 45, 20, 0},
			{nil, -45, 10, 15},
			{nil, 45, 10, -15},
			{nil, -90, 10, 0},
			{nil, 90, 10, 0},
			{nil, -180, 20, 0},
			{nil, -135, 15, 0},
			{nil, 135, 15, 0},
		},
	}

	function spaceship.load()
		spaceship.image = love.graphics.newImage("media/spaceship.png")
		spaceship.saveState()
	end

	function spaceship.create() 
		local ship = {
			x = 0,
			y = 0,
			vx = 220.0,
			vy = 0.0,
			angle = 0,
			accUpTime = 0,
		}
		spaceship.instance = spaceship.instance or ship
		return ship
	end

	function spaceship.reset()
		spaceship.instance = spaceship.create()
	end

	function spaceship.saveState()
		spaceship.storedState = copyTable(spaceship.weaponSpots)
	end

	function spaceship.loadState()
		spaceship.weaponSpots = copyTable(spaceship.storedState)
	end

	function spaceship.update(ship) 
		ship = ship or spaceship.instance
		-- Gravity
		ship.vy = ship.vy + spaceship.gravity * gameState.dt
		-- Upwards acceleration
		if ship.accUpTime > 0 then
			ship.accUpTime = ship.accUpTime - gameState.dt
			ship.vy = ship.vy - 12 * spaceship.gravity * gameState.dt
		end
		-- Movement
		ship.x = ship.x + gameState.dt * ship.vx
		ship.y = ship.y + gameState.dt * ship.vy
		-- Apply angle
		ship.angle = math.atan2(ship.vy * 0.1, ship.vx)
		-- Bounds
		ship.y = gameState.wrapY(ship.y)
		-- Shoot
		for i = 1,#spaceship.weaponSpots do
			if spaceship.weaponSpots[i][1] ~= nil then
				local tp = spaceship.weaponSpots[i][1]
				if gameState.periodicCheck(tp.delay) then
					bullets.create(ship, spaceship.weaponSpots[i], tp)
				end
			end
		end
		-- Pickups
		if pickups.sphereCollision(ship.x, ship.y, 40) then
			pickups.apply(pickups.collider, ship)
			lush.play("powerup.wav")
		end
		-- Collision with Asteroid
		if asteroid.sphereCollision(ship.x, ship.y, 20) then
			explosion.create(ship.x, ship.y, 1.6)
			gameState.resetLevel()
			return
		end
		-- Collision with enemies
		if enemy.sphereCollision(ship.x, ship.y, 20) then
			explosion.create(ship.x, ship.y, 1.6)
			gameState.resetLevel()
			return
		end
	end

	function spaceship.draw(ship)
		love.graphics.setColor(255,255,255)
		ship = ship or spaceship.instance
		-- line
		--local dx = math.cos(ship.angle)
		--local dy = math.sin(ship.angle)
		--love.graphics.line(ship.x - 20*dx, ship.y - 20*dy, ship.x + 40*dx, ship.y + 40*dy)
		love.graphics.draw(spaceship.image, ship.x, ship.y, ship.angle, 0.35, 0.35, 0.4*512, 0.5*512)
	end

	function spaceship.handleKeypress(key, ship)
		ship = ship or spaceship.instance
		if key == "f" then
			ship.accUpTime = 0.12
		end
		if key == "h" then
			lush.play("honk.wav")
		end
	end


	function spaceship.enhanceWeapon(num)
		-- find free weapon spot
		for i = 1,#spaceship.weaponSpots do
			if spaceship.weaponSpots[i][1] == nil then
				-- apply
				spaceship.weaponSpots[i][1] = bullets.types[num]
				return true
			end
		end
		return false
	end



end